@@ -47,18 +47,26 @@
 
 #undef _BRAILLE_DEBUG_
 
-gchar *src_braille_device 	  = NULL;
-gint  src_braille_port 		  = -1;
-gint  src_braille_position_sensor = -1;
-gint  src_braille_optical_sensor  = -1;
-gint  src_braille_offset 	  = 0;
-gchar *src_braille_style 	  = NULL;
-gchar *src_braille_cursor_style   = NULL;
-gchar *src_braille_translation_table = NULL;
+typedef struct
+{
+    gchar	 *device;
+    gchar 	 *style;
+    gchar 	 *cursor_style;	
+    gchar 	 *translation_table;
+    
+    gint  	 port;
+    gint  	 position_sensor;
+    gint  	 optical_sensor;
+    gint  	 offset;
+
+}SRCBraille;
+
+static SRCBraille *src_braille = NULL;
 
 extern gboolean src_use_braille;
 extern gboolean src_use_braille_monitor;
-extern void brl_input_event (BrlInEvent *event);
+extern gchar* scroll;
+extern void brl_input_event (BRLInEvent *event);
 extern gchar *fmt_for_xml (gchar *, gchar *, gchar *);
 
 gint 	brlmon_sd, 	/* BrlMonitor socket descriptor*/
@@ -93,12 +101,13 @@ static gboolean brlmonsockstate = FALSE;
 static LincSockLen remote_brlmon_addr_len;
 static LincSockLen remote_braille_addr_len;
 
+
 static gboolean 
 udp_sock_glib_cb(GIOChannel *source, 
 		GIOCondition condition, 
 		gpointer data)
 {	
-    BrlInEvent event;
+    BRLInEvent event;
     BrlPackage package;
     gint clilen;
     gint n;
@@ -530,17 +539,29 @@ src_brlmon_show (gchar *message)
     gchar offset[5];
 	
     text = src_xml_format ("TEXT", NULL, message);
-    sprintf (offset, "%d", src_braille_offset);
+    sprintf (offset, "%d", src_braille->offset);
     brlout = g_strconcat (
-		"<BRLOUT language=\"", src_braille_translation_table,"\" bstyle=\"", src_braille_style ,"\" clear=\"true\">\n",
+		"<BRLOUT language=\"", src_braille->translation_table,"\" bstyle=\"", src_braille->style ,"\" clear=\"true\">\n",
 		/* main display */
 		"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\">\n",
 		text,
-		"</BRLDISP>\n",
-		/* nothing to do on state display here*/
-		"</BRLOUT>\n", 
 		NULL);
-
+    if (scroll && scroll[0])
+    {
+	brlout = g_strconcat (brlout,
+	                      "<SCROLL mode=\"", scroll, "\" /SCROLL>\n",
+		              "</BRLDISP>\n",
+		              "</BRLOUT>\n",
+		              NULL);
+    }
+    else
+    {		
+	brlout = g_strconcat (brlout,
+		              "</BRLDISP>\n",
+		              "</BRLOUT>\n", 
+    		              NULL);
+    }		    
+    		
     src_brlmon_send  (brlout);
     g_free (brlout);						
     g_free(text);
@@ -568,9 +589,9 @@ src_braille_show (gchar *message)
     gchar offset[5];
 	
     text = src_xml_format ("TEXT", NULL, message);
-    sprintf (offset, "%d", src_braille_offset);	
+    sprintf (offset, "%d", src_braille->offset);	
     brlout = g_strconcat (
-		"<BRLOUT language=\"",src_braille_translation_table,"\" bstyle=\"", src_braille_style,"\" clear=\"true\">\n",
+		"<BRLOUT language=\"",src_braille->translation_table,"\" bstyle=\"", src_braille->style,"\" clear=\"true\">\n",
 		/* main display */
 		"<BRLDISP role=\"main\" offset=\"", offset, "\" clear=\"true\">\n",
 		text,
@@ -616,13 +637,13 @@ src_braille_restart ()
 {
     if (src_use_braille)
     {
-	gchar *device = g_strdup (src_braille_device);
-	gint port = src_braille_port;
+	gchar *device = g_strdup (src_braille->device);
+	gint port = src_braille->port;
 	/* terminate Braille */
 	src_braille_terminate ();
     	/* and then start it again */
-	src_braille_device = device;
-	src_braille_port = port;	
+	src_braille->device = device;
+	src_braille->port = port;	
 	src_use_braille = src_braille_init ();
     }
     if (src_use_braille_monitor)
@@ -702,52 +723,115 @@ src_braille_monitor_init ()
     return TRUE;
 }
 
+static SRCBraille*
+src_braille_setting_new ()
+{
+    SRCBraille *braille = NULL;
+    
+    braille = g_new0 (SRCBraille, 1);
+    
+    return braille;
+}
+
+static void
+src_braille_load_values (SRCBraille *braille)
+{
+    sru_assert (braille);
+    
+    if (!braille->device)
+    {
+	gchar *default_device = DEFAULT_BRAILLE_DEVICE;
+	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_DEVICE, CFGT_STRING, 
+					      &braille->device, 
+					      default_device);
+    }
+        
+    if (MIN_BRAILLE_PORT > braille->port || 
+	MAX_BRAILLE_PORT < braille->port)
+    {
+	gint default_port = DEFAULT_BRAILLE_PORT_NO;
+	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_PORT_NO, CFGT_INT, 
+					      &braille->port, 
+					      &default_port);
+    
+	if (MIN_BRAILLE_PORT > braille->port || 
+	    MAX_BRAILLE_PORT < braille->port) 
+	{
+    	    braille->port = DEFAULT_BRAILLE_PORT_NO;
+    	    SET_BRAILLE_CONFIG_DATA (BRAILLE_PORT_NO, CFGT_INT, &braille->port);
+	}					  
+    }    
+    
+    gchar *default_braille_transaltion_table = DEFAULT_BRAILLE_TRANSLATION;
+    GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_TRANSLATION, CFGT_STRING, 
+					  &braille->translation_table, 
+					  default_braille_transaltion_table);		
+    if (!src_braille_translation_table_exist (braille->translation_table))
+    {
+        srconf_set_config_data (BRAILLE_TRANSLATION, CFGT_STRING, 
+				DEFAULT_BRAILLE_TRANSLATION, 
+				CFGM_BRAILLE);
+    }
+	
+    
+    gchar *default_braille_style = DEFAULT_BRAILLE_STYLE;
+    GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_STYLE, CFGT_STRING, 
+					  &braille->style, 
+					  default_braille_style);
+					  
+    gchar *default_braille_cursor_style = DEFAULT_BRAILLE_CURSOR_STYLE;
+    GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_CURSOR_STYLE, CFGT_STRING, 
+					  &braille->cursor_style, 
+					  default_braille_cursor_style);
+					  
+    gint default_pos_sensor = DEFAULT_BRAILLE_POSITION_SENSOR;
+    GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_POSITION_SENSOR, CFGT_INT, 
+					  &braille->position_sensor, 
+					  &default_pos_sensor);
+    
+    gint default_opt_sensor = DEFAULT_BRAILLE_OPTICAL_SENSOR;
+    GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_OPTICAL_SENSOR, CFGT_INT, 
+					  &braille->optical_sensor, 
+					  &default_opt_sensor);
+	
+}
+
+void
+src_braille_get_defaults ()
+{
+    src_braille = src_braille_setting_new ();
+    src_braille_load_values (src_braille);
+}
+
 gboolean 
 src_braille_init ()
 {
     gboolean brlval = FALSE;
-    
     brl_in_xml_init (brl_input_event);
-    if (!src_braille_device)
-    {
-	gchar *default_device = DEFAULT_BRAILLE_DEVICE;
-	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_DEVICE, CFGT_STRING, 
-		&src_braille_device, default_device);
-    }
-	
-    if (MIN_BRAILLE_PORT > src_braille_port || 
-	MAX_BRAILLE_PORT < src_braille_port) /* just for sure */
-    {
-	gint default_port = DEFAULT_BRAILLE_PORT_NO;
-	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_PORT_NO, CFGT_INT, 
-		    &src_braille_port, &default_port);
-		    
-	if (MIN_BRAILLE_PORT > src_braille_port || MAX_BRAILLE_PORT < src_braille_port) 
-	{
-	    src_braille_port = DEFAULT_BRAILLE_PORT_NO;
-	    SET_BRAILLE_CONFIG_DATA (BRAILLE_PORT_NO, CFGT_INT, &src_braille_port);
-	}
-    }
+    
+    sru_assert (src_braille && src_braille->device);
 
-    if (!brl_xml_init(src_braille_device, src_braille_port, brl_xml_input_proc)) /* take Braille display from Gnopi... */
+    if (!brl_xml_init(src_braille->device, src_braille->port, brl_xml_input_proc)) /* take Braille display from Gnopi... */
     {
 	brlval = FALSE;
 	fprintf(stderr, "\n** BRAILLE initialization failed for %s on port %d",
-					src_braille_device, src_braille_port);
+					src_braille->device, src_braille->port);
 	fprintf (stderr, "\n   To change one or more settings use gnopernicus UI\n");
     }
     else
     {
 	brlval = TRUE;
 	fprintf (stderr, "\n** BRAILLE initialization succeded for %s device"
-				    " on port %d.", src_braille_device, src_braille_port);
+				    " on port %d.", src_braille->device, src_braille->port);
 	fprintf (stderr, "\n   To see something you must have a Braille device"
 				    " connected to your computer.");
 	fprintf (stderr, "\n   To change one or more settings use gnopernicus UI.\n");
 	
     }
+    
     srconf_unset_key (SRCORE_BRAILLE_SENSITIVE, SRCORE_PATH);
     SET_SRCORE_CONFIG_DATA (SRCORE_BRAILLE_SENSITIVE, CFGT_BOOL, &brlval);
+    
     return brlval;
 }
 
@@ -764,8 +848,253 @@ src_braille_terminate ()
     brl_xml_terminate ();
     brl_in_xml_terminate ();
 	
-    if (src_braille_device)
-	g_free (src_braille_device);
-    src_braille_device = NULL;
-    src_braille_port = -1;
+    g_free (src_braille->device);
+    src_braille->port = -1;
+}
+
+static SRCBraille* 
+src_braille_get_global_settings ()
+{
+    sru_assert (src_braille);
+    return src_braille;
+}
+
+gboolean
+src_braille_set_device (gchar *device)
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille);
+    sru_assert (device && braille && braille->device);
+    
+    if (strcmp (device, braille->device) != 0)
+    {
+	g_free (braille->device);
+	braille->device = g_strdup (device);
+	src_use_braille = src_braille_restart ();
+	return TRUE ;
+    }
+    return FALSE;
+}
+
+gboolean
+src_braille_set_port_no (gint port)
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille);
+    
+    if (braille->port != port)
+    {
+	braille->port = port;
+	src_use_braille = src_braille_restart ();
+	return TRUE;
+    }    
+    return FALSE;
+}
+
+gboolean
+src_braille_set_style (gchar *style)
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille);
+    sru_assert (style && braille->style);
+    
+    if (strcmp (style, braille->style) != 0)
+    {
+	g_free (braille->style);
+	braille->style = g_strdup (style);
+	
+	return TRUE;
+    }
+    return FALSE;
+}
+
+gboolean
+src_braille_set_cursor_style (gchar *cursor_style)
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille);
+    sru_assert (cursor_style && braille->cursor_style);
+    
+    if (strcmp (cursor_style, braille->cursor_style) != 0)
+    {
+	g_free (braille->cursor_style);
+	braille->cursor_style = g_strdup (cursor_style);
+	
+	return TRUE;
+    }
+    return FALSE;
+}
+
+gboolean
+src_braille_set_translation_table (gchar *translation_table)
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille);
+    
+    sru_assert (translation_table && braille->translation_table);
+    if (strcmp (translation_table, braille->translation_table) != 0)
+    {
+	if (src_braille_translation_table_exist (translation_table))
+	{
+	    g_free (braille->translation_table);
+	    braille->translation_table = g_strdup (translation_table);
+	
+	    src_use_braille = src_braille_restart ();
+	}
+	else
+	{
+	    gchar *value = DEFAULT_BRAILLE_TRANSLATION;
+	    srconf_set_config_data (BRAILLE_TRANSLATION,
+			            CFGT_STRING,
+			            value,
+			    	    CFGM_BRAILLE);
+	}
+    }
+    return FALSE; 
+}
+
+gboolean
+src_braille_set_position_sensor (gint position_sensor)
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille);
+    
+    if (position_sensor != braille->position_sensor)
+    {
+	braille->position_sensor = position_sensor;
+    }
+
+    return TRUE;
+}
+
+gboolean
+src_braille_set_optical_sensor (gint optical_sensor)
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille);
+    
+    if (optical_sensor != braille->optical_sensor)
+    {
+	braille->optical_sensor = optical_sensor;
+    }
+
+    return TRUE;
+}
+
+gboolean
+src_braille_set_offset (gint offset)
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille);
+    
+    if (offset != braille->offset)
+    {
+	braille->offset = offset;
+    }
+
+    return TRUE;
+}
+
+gint
+src_braille_get_offset ()
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille);
+    
+    return braille->offset;
+}
+
+gint
+src_braille_get_optical_sensor ()
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille);
+    
+    return braille->optical_sensor;
+}
+
+gint
+src_braille_get_position_sensor ()
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille);
+    
+    return braille->position_sensor;
+}
+
+gint
+src_braille_get_port_no ()
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille);
+    
+    return braille->port;
+}
+
+gchar*
+src_braille_get_translation_table ()
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille && braille->translation_table);
+    
+    return braille->translation_table;
+}
+
+gchar*
+src_braille_get_cursor_style ()
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille && braille->cursor_style);
+    
+    return braille->cursor_style;
+}
+
+gchar*
+src_braille_get_style ()
+{
+    SRCBraille *braille;
+    
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille && braille->style);
+    
+    return braille->style;
+}
+
+gchar*
+src_braille_get_device ()
+{
+    SRCBraille *braille;
+        
+    braille = src_braille_get_global_settings ();
+    sru_assert (braille && braille->device);
+    
+    return braille->device;
 }
