@@ -55,6 +55,9 @@ gint src_caret_position;
 gchar *braille_device = NULL;
 gint   braille_port;
 
+extern gchar *scroll;
+extern gint old_offset;
+
 extern gboolean panning;
 
 
@@ -105,11 +108,13 @@ src_present_real_sro_for_brl_and_brlmon (SRObject *obj, gchar *reason)
 	gchar *braille_translation_table = NULL;
 	gint  braille_offset;
 	gchar offset[5];
+	gchar *braille_scroll = NULL;
 
 	braille_style = src_braille_get_style ();
 	braille_cursor_style = src_braille_get_cursor_style ();
 	braille_translation_table = src_braille_get_translation_table ();
 	braille_offset = src_braille_get_offset();
+	braille_scroll = g_strdup (scroll);
 	
 	sro_get_role (obj, &role, SR_INDEX_CONTAINER);
 	if (role == SR_ROLE_TEXT_SL || role == SR_ROLE_TEXT_ML || role == SR_ROLE_TERMINAL || role == SR_ROLE_COMBO_BOX)
@@ -167,10 +172,25 @@ src_present_real_sro_for_brl_and_brlmon (SRObject *obj, gchar *reason)
 			cursor ? cursor : "",
 			">",
 			txt,
-			"</BRLDISP>",
-			status ? status : "",
-			"</BRLOUT>",
 			NULL);
+	if (braille_scroll && braille_scroll[0])
+	{
+	    tmp = g_strconcat (tmp, 
+			       "<SCROLL mode=\"", braille_scroll, "\"></SCROLL>", 
+			       "</BRLDISP>",
+			       status ? status : "",
+	                       "</BRLOUT>", 
+			       NULL);		       
+	    g_free (braille_scroll);
+	}		
+	else
+	{
+	    tmp = g_strconcat (tmp, 
+			       "</BRLDISP>",
+			       status ? status : "",
+	                       "</BRLOUT>", 
+			       NULL);
+	}		
 	/*fprintf (stderr, "\n%s", tmp);*/
 	if (src_use_braille)
 	    src_braille_send (tmp);
@@ -287,7 +307,19 @@ src_present_real_sro (SRObject *obj)
 	return;
     
     if (old != obj)
+    {	
 	src_braille_set_offset (0);
+	old_offset = 0;
+	if (scroll)
+	{
+	    g_free (scroll);
+	    scroll = NULL;
+        }
+     }
+    else
+    {
+	src_braille_set_offset (old_offset);
+    }
     old = obj;
 
     if (!src_use_braille && !src_use_speech && !src_use_magnifier && !src_use_braille_monitor)
@@ -2044,7 +2076,7 @@ src_event_sink(SREvent *event_,
 }
 
 void
-brl_input_event (BrlInEvent *event)
+brl_input_event (BRLInEvent *event)
 {
     switch (event->event_type)
     {
